@@ -1,5 +1,5 @@
 ;;; Guile-Avahi --- Guile bindings for Avahi.
-;;; Copyright (C) 2007  Ludovic Courts <ludo@gnu.org>
+;;; Copyright (C) 2007, 2008  Ludovic Courts <ludo@gnu.org>
 ;;;
 ;;; This file is part of Guile-Avahi.
 ;;;
@@ -26,6 +26,9 @@
 
            make-guile-poll guile-poll
            make-simple-poll simple-poll iterate-simple-poll run-simple-poll
+           make-threaded-poll threaded-poll
+           start-threaded-poll stop-threaded-poll quit-threaded-poll
+           lock-threaded-poll unlock-threaded-poll
 
            watch-event->string
            watch-event/in watch-event/out watch-event/err watch-event/hup
@@ -90,11 +93,13 @@
            error/not-supported
            error/not-permitted
            error/invalid-argument
-           error/is-empty
-           error/no-change))
+           error/is-empty))
 
 (load-extension "libguile-avahi-v-0" "scm_avahi_common_init")
 
+;; Optional bindings, depending on the configuration.
+(if (defined? 'error/no-change) (export error/no-change))
+
 ;; Aliases.
 (define protocol/unspecified    protocol/unspec)
 (define interface/unspecified   interface/unspec)
