@@ -1,5 +1,5 @@
 /* Guile-Avahi --- Guile bindings for Avahi.
-   Copyright (C) 2007  Ludovic Courts <ludo@gnu.org>
+   Copyright (C) 2007, 2008  Ludovic Courts <ludo@gnu.org>
 
    This file is part of Guile-Avahi.
 
@@ -17,7 +17,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifdef HAVE_CONFIG_H
-# include "config.h"
+# include <config.h>
 #endif
 
 #ifdef HAVE_GMP_H
